@@ -2,7 +2,7 @@
 This file is part of bruteforce-salted-openssl, a program trying to
 bruteforce a file encrypted (with salt) by openssl.
 
-Copyright 2014-2017 Guillaume LE VAILLANT
+Copyright 2014-2018 Guillaume LE VAILLANT
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -78,7 +78,7 @@ FILE *dictionary = NULL;
 const EVP_CIPHER *cipher = NULL;
 const EVP_MD *digest = NULL;
 pthread_mutex_t found_password_lock, get_password_lock;
-char stop = 0, only_one_password = 0, found_password = 0, no_error = 0;
+char stop = 0, only_one_password = 0, found_password = 0, no_error = 0, no_salt = 0;
 unsigned int nb_threads = 1;
 unsigned long long int limit = 0, count_limit = 0;
 unsigned char last_pass[LAST_PASS_MAX_SHOWN_LENGTH];
@@ -106,6 +106,7 @@ void handle_signal(int signo)
   double space = 0;
   double pw_per_seconds;
   time_t current_time, eta_time;
+  struct tm *time_info;
   char datestr[256];
 
   current_time = time(NULL);
@@ -139,8 +140,11 @@ void handle_signal(int signo)
     }
     else
     {
-      strftime(datestr, 256, "%c", localtime(&eta_time));
-      fprintf(stderr, "ETA: %s\n", datestr);
+      time_info = localtime(&eta_time);
+      if(time_info && (strftime(datestr, 256, "%c", time_info) > 0))
+        fprintf(stderr, "ETA: %s\n", datestr);
+      else
+        fprintf(stderr, "ETA: %ld s\n", eta_time - start_time);
     }
   }
   fprintf(stderr, "\n");
@@ -415,7 +419,10 @@ void * decryption_func(void *arg)
       break;
 
     /* Decrypt data with password */
-    EVP_BytesToKey(cipher, digest, salt, pwd, pwd_len, 1, key, iv);
+    if(no_salt)
+      EVP_BytesToKey(cipher, digest, NULL, pwd, pwd_len, 1, key, iv);
+    else
+      EVP_BytesToKey(cipher, digest, salt, pwd, pwd_len, 1, key, iv);
     EVP_DecryptInit(ctx, cipher, key, iv);
     EVP_DecryptUpdate(ctx, out, &out_len1, data, data_len);
     ret = EVP_DecryptFinal(ctx, out + out_len1, &out_len2);
@@ -527,10 +534,12 @@ void restore_state()
 
   if(state == NULL)
   {
-    fprintf(stderr, "Warning: can't open state file, state not restored.\n\n");
+    fprintf(stderr, "Warning: can't open state file, state not restored, a new file will be created.\n\n");
     return;
   }
 
+  fprintf(stderr, "Warning: restoring state, ignoring options -B, -b, -e, -f, -l, -m and -s.\n\n");
+
   if(dictionary != NULL)
     fclose(dictionary);
 
@@ -766,6 +775,7 @@ void usage(char *progname)
   fprintf(stderr, "  -m <length>  Maximum password length (beginning and end included).\n");
   fprintf(stderr, "                 default: 8\n\n");
   fprintf(stderr, "  -N           Ignore decryption errors (similar to openssl -nopad).\n\n");
+  fprintf(stderr, "  -n           Ignore salt (similar to openssl -nosalt).\n\n");
   fprintf(stderr, "  -s <string>  Password character set.\n");
   fprintf(stderr, "               default: \"0123456789ABCDEFGHIJKLMNOPQRSTU\n");
   fprintf(stderr, "                         VWXYZabcdefghijklmnopqrstuvwxyz\"\n\n");
@@ -801,7 +811,7 @@ int main(int argc, char **argv)
 
   /* Get options and parameters */
   opterr = 0;
-  while((c = getopt(argc, argv, "1aB:b:c:d:e:f:hL:l:M:m:Ns:t:v:w:")) != -1)
+  while((c = getopt(argc, argv, "1aB:b:c:d:e:f:hL:l:M:m:Nns:t:v:w:")) != -1)
     switch(c)
     {
     case '1':
@@ -901,6 +911,10 @@ int main(int argc, char **argv)
       no_error = 1;
       break;
 
+    case 'n':
+      no_salt = 1;
+      break;
+
     case 's':
       charset_len = mbstowcs(NULL, optarg, 0);
       if(charset_len == 0)
@@ -1055,27 +1069,33 @@ int main(int argc, char **argv)
     perror("open file");
     exit(EXIT_FAILURE);
   }
-  memset(salt, 0, sizeof(salt));
-  ret = read(fd, salt, 8);
-  if(strncmp(salt, "Salted__", 8) != 0)
+  if(no_salt == 0)
   {
-    close(fd);
-    fprintf(stderr, "Error: %s is not a salted openssl file.\n\n", filename);
-    exit(EXIT_FAILURE);
-  }
+    memset(salt, 0, sizeof(salt));
+    ret = read(fd, salt, 8);
+    if(strncmp(salt, "Salted__", 8) != 0)
+    {
+      close(fd);
+      fprintf(stderr, "Error: %s is not a salted openssl file.\n\n", filename);
+      exit(EXIT_FAILURE);
+    }
 
-  /* Read salt */
-  ret = read(fd, salt, 8);
-  if(ret != 8)
-  {
-    close(fd);
-    fprintf(stderr, "Error: could not read salt.\n\n");
-    exit(EXIT_FAILURE);
+    /* Read salt */
+    ret = read(fd, salt, 8);
+    if(ret != 8)
+    {
+      close(fd);
+      fprintf(stderr, "Error: could not read salt.\n\n");
+      exit(EXIT_FAILURE);
+    }
   }
 
   /* Read encrypted data */
   ret = fstat(fd, &file_stats);
-  data_len = file_stats.st_size - 16;
+  if(no_salt)
+    data_len = file_stats.st_size;
+  else
+    data_len = file_stats.st_size - 16;
   data = (char *) malloc(data_len);
   if(data == NULL)
   {
@@ -1122,8 +1142,6 @@ int main(int argc, char **argv)
 
   if(state_file != NULL)
   {
-    fprintf(stderr, "Warning: restoring state, ignoring options -B, -b, -e, -f, -l, -m and -s.\n\n");
-
     restore_state();
 
     signal(SIGVTALRM, save_state);
@@ -1152,7 +1170,10 @@ int main(int argc, char **argv)
   if(found_password == 0)
   {
     handle_signal(SIGUSR1); /* Print some stats */
-    fprintf(stderr, "Password not found\n");
+    fprintf(stderr, "Password not found.\n");
+    fprintf(stderr, "The file might have been encrypted with a different cipher or/and a\n");
+    fprintf(stderr, "different digest (e.g. OpenSSL 1.0.x uses the MD5 digest by default\n");
+    fprintf(stderr, "but OpenSSL 1.1.x uses SHA256 by default).\n");
   }
 
   free(thread_locals);
